import os
from datetime import datetime
import argparse

# Issue a reminder to stdout after every N lines...
# (just to re-assure the user we're doing something on very large files)
# On a 2.7GHz i7 we should see around 13Million node lines/min
# and about 9Million edge lines/min.
# Set to 0 for quiet operation.
LINE_REMINDER = 0


def do_for_dir(input_dir):
    os.chdir(input_dir)
    prop_dict = {
        "nodes.txt": [None, "smiles:ID(F2)", "hac:INT", "chac:INT", "osmiles"],
        "edges.txt": [None, ":START_ID(F2)", ":END_ID(F2)", "label"],
    }
    # build a map of SMILES to chemical ID
    # from the attributes file...
    attrs = {}
    with open("attributes.txt") as attr_f:
        for line in attr_f:
            line_parts = line.split()
            attrs[line_parts[1]] = line_parts[3]

    for f_name in prop_dict:

        if LINE_REMINDER:
            print('%s Processing %s/%s...' %
                  (datetime.now(), input_dir, f_name))
        line_num = 0

        out_f = open(f_name.replace(".txt", ".csv"), "w")
        with open(f_name) as in_f:
            for line in in_f:

                if LINE_REMINDER:
                    line_num += 1
                    if line_num % LINE_REMINDER == 0:
                        print('%s ...%d' % (datetime.now(), line_num))

                line_spl = line.split()
                out_l = []
                for i, x in enumerate(prop_dict[f_name]):
                    if x:
                        out_l.append(line_spl[i])
                if f_name == "nodes.txt":
                    attr_cmp_id = attrs.get(line_spl[1], None)
                    if attr_cmp_id:
                        cmpd_id = attr_cmp_id
                        out_l.append(cmpd_id)
                        # This is where we can add tags - like CHEAP - EXPENSIVE
                        out_l.append("EM;MOL;F2")
                    else:
                        out_l.append("")
                        out_l.append("F2")
                    out_f.write(",".join(out_l) + "\n")
                elif f_name == "edges.txt":
                    out_f.write(",".join(out_l) + "\n")

        if LINE_REMINDER:
            print('%s Processed (%d)' % (datetime.now(), line_num))
        out_f.flush()
        out_f.close()


def main():
    parser = argparse.ArgumentParser(
        description="Convert text files output from build_db into CSV files for neo4j."
    )
    parser.add_argument("--input_dir")
    args = parser.parse_args()

    node_list = ["nodes-header.csv"]
    edge_list = ["edges-header.csv"]
    with open("edges-header.csv", "w") as out_f:
        out_f.write(",".join([x for x in [":START_ID(F2)", ":END_ID(F2)", "label"]]))
    with open("nodes-header.csv", "w") as out_f:
        out_f.write(
            ",".join(
                [
                    x
                    for x in [
                        "smiles:ID(F2)",
                        "hac:INT",
                        "chac:INT",
                        "osmiles",
                        "cmpd_id",
                        ":LABEL",
                    ]
                    if x
                ]
            )
            + "\n"
        )
    do_for_dir(args.input_dir)
    node_list.append("nodes.csv")
    edge_list.append("edges.csv")
    # Now this is the command to run
    print(
        " ".join(
            [
                "/var/lib/neo4j/bin/neo4j-admin import",
                "--database",
                "new.db",
                "--nodes",
                '"' + ",".join(node_list) + '"',
                "--relationships:F2EDGE",
                '"' + ",".join(edge_list) + '"',
            ]
        )
    )


if __name__ == "__main__":
    main()



# Series of functions to parse input files
from frag.alysis.models import Object, Owner
from frag.utils.rdkit_utils import (
    _parse_ligand_sdf,
    _get_c_of_mass,
    RDKitPh4,
    RDKitAtom,
    _get_water_coords,
    _get_waters,
    _get_res,
    _get_res_rmsds,
    _parse_pdb,
)
from rdkit import Chem


def _get_c_of_mass_list(mols):
    c_of_mass_list = []
    for m in mols:
        c_of_mass_list.append(_get_c_of_mass(m))
    return c_of_mass_list


def parse_ligands(input_file, input_type="sdf"):
    mols = _parse_ligand_sdf(input_file=input_file)
    # Now return them with their name and centre of mass
    return _get_c_of_mass_list(mols)


def parse_ligand_ph4s(input_mols):
    """
    Function to return a series of ligand based pharmacophores.
    :param input_mols: the RDKit molecules
    :return: the molecule based pharmacophores
    """
    rdkit_ph4 = RDKitPh4()
    rdkit_atom = RDKitAtom()
    output_pharma_list = []
    for mol in input_mols:
        if not mol:
            pharma_list = []
        else:
            pharma_list = rdkit_ph4.generate_ph4_for_mol(rdmol=mol)
            atom_list = rdkit_atom.generate_atoms_for_mol(mol)
            x, y, z = _get_c_of_mass(rdmol=mol)
            c_of_m_feat = (x, y, z, "c_of_m")
            pharma_list.append(c_of_m_feat)
            pharma_list.extend(atom_list)
        output_pharma_list.append(pharma_list)
    return output_pharma_list


def parse_waters(input_pdbs, input_mol=None, max_dist=10.0):
    """
    Function to parse a series of waters - return waters.
    :param input_pdb: the input PDB files
    :param input_mol: the input molecule (to use as a reference around which to limit)
    :return: tuple threes of coordinates of the waters
    """
    output_water_list = []
    # First just get the waters from the file
    for input_pdb in input_pdbs:
        waters = _get_waters(open(input_pdb).readlines())
        output_water_list.append(_get_water_coords(waters))
    return output_water_list


def parse_residues(input_pdbs, input_mol=None, max_dist=10.0):
    """
    Function to parse a series of PDB files of proteins.
    :param input_pdb: the input PDB files - with identifiers
    :return: a dict (Key Residue -> value list of molecules)
    """
    owner_list = []
    res_dict = {}
    for input_pdb in input_pdbs:
        # Loop through the residues
        mol = _parse_pdb(input_pdb)
        this_res_dict = _get_res(mol)
        for key in this_res_dict:
            if key in res_dict:
                res_dict[key].append(res_dict[key])
            else:
                res_dict[key] = [res_dict[key]]
    for res in res_dict:
        rmsd_coords = _get_res_rmsds(res_dict[res])
        out_l = []
        res = Object(rmsd_coords, res)
        out_l.append(res)
        owner = Owner(out_l, input_pdb)
        owner_list.append(owner)
    return owner_list


def get_file(file_path, output_format, file_counter):
    if output_format == "smi":
        return Chem.SmilesWriter(file_path + "_" + str(file_counter) + ".smi")
    else:
        return Chem.SDWriter(file_path + "_" + str(file_counter) + ".sdf")


def parse_mols(input_file, input_format):
    if input_format == "smi":
        return Chem.SmilesMolSupplier(input_file)
    else:
        return Chem.SDMolSupplier(input_file)



from frag.network.decorate import decorate_smi
import argparse
from rdkit import Chem
from rdkit.Chem.Scaffolds import MurckoScaffold
from tqdm import tqdm
from frag.network.models import Attr

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Decorate a library of molecules for insertion to the database.')
    parser.add_argument('--input_smi')
    parser.add_argument('--output_attr')
    args = parser.parse_args()
    out_smi = open(args.output_attr,"w")
    for mol in tqdm(Chem.SmilesMolSupplier(args.input_smi,delimiter=',',smilesColumn=1,nameColumn=0)):
        this_smi = Chem.MolToSmiles(mol,isomericSmiles=True)
        new_smis = decorate_smi(this_smi)
        new_murck = decorate_smi(MurckoScaffold.MurckoScaffoldSmiles(this_smi))
        # mol_frags = get_fragments(Chem.MolFromSmiles(this_smi),iso_labels=False)
        # new_smis.extend([x.replace("Xe","At") for x in mol_frags])
        new_smis.extend(new_murck)
        new_smis = list(set(new_smis))
        # Do this on original and on Murcko Scaffold
        name = mol.GetProp("_Name")
        new_attr = Attr(this_smi, ["EM", name])
        out_smi.write(str(new_attr) + "\n")
        for i,smi in enumerate(new_smis):
            new_attr = Attr(smi,["EM",name+"_"+str(i)])
            out_smi.write(str(new_attr)+"\n")


import unittest

from rdkit import Chem

from frag.network.models import NodeHolder, Node, Attr
from frag.utils.network_utils import (
    rebuild_smi,
    make_child_mol,
    get_fragments,
    build_network,
    get_comb_index,
    ret_comb_index,
)
from frag.network.decorate import (
    decorate_smi,
    deletion_linker_mol,
    deletion_linker_smi,
    del_link_coord,
)


def parse_node(input_str):
    """
    Convert something like to a Node:
    NODE O=CCCc1ccc(cc1)c2ccccc2 16 12 OCCCC1CCC(CC1)C2CCCCC2 0
    :param input_str:
    :return:
    """
    smiles = input_str.split()[1]
    new_node = Node()
    new_node.SMILES = Chem.CanonSmiles(smiles)
    new_node.HAC = input_str.split()[2]
    new_node.RAC = input_str.split()[3]
    new_node.RING_SMILES = input_str.split()[4]
    return new_node


def conv_smi(input_smi):
    return Chem.MolToSmiles(Chem.MolFromSmiles(input_smi), isomericSmiles=False)


class NetworksTest(unittest.TestCase):

    def test_rebuild(self):
        input_list = [
            ["O[100Xe]", "[100Xe]c1ccc([101Xe])cc1"],
            ["O[100Xe]", "[101Xe]c1ccccc1"],
            ["[101Xe]c1ccccc1", "[100Xe]c1ccc([101Xe])cc1"],
        ]
        rebuild_list = [
            "Oc1ccc([Xe])cc1",
            "O[Xe].[Xe]c1ccccc1",
            "[Xe]c1ccc(cc1)c2ccccc2",
        ]
        for i in range(len(input_list)):
            self.assertEqual(
                conv_smi(rebuild_smi(input_list[i], ring_ring=False)),
                conv_smi(rebuild_list[i]),
            )

    def test_child(self):
        rebuild_list = [
            "Oc1ccc([Xe])cc1",
            "O[Xe].[Xe]c1ccccc1",
            "[Xe]c1ccc(cc1)c2ccccc2",
        ]
        child_list = ["Oc1ccccc1", "O.c1ccccc1", "c1ccc(cc1)c2ccccc2"]
        for i in range(len(child_list)):
            self.assertEqual(
                conv_smi(make_child_mol(rebuild_list[i])), conv_smi(child_list[i])
            )

    def test_get(self):
        input_list = ["CC.CC", "CC.c1ccccc1C", "CCC"]
        output_list = [["CC", "CC"], ["CC", "[100Xe]C", "[100Xe]c1ccccc1"], ["CCC"]]
        for i in range(len(input_list)):
            self.assertListEqual(
                output_list[i], get_fragments(Chem.MolFromSmiles(input_list[i]))
            )

    def test_generate_nodes(self):
        """
        Test we can generate nodes for the basic data.
        :return:
        """
        try:
            nodes = [x for x in open("frag/tests/data/nodes.txt").readlines()]
            edges = [x.split() for x in open("frag/tests/data/edges.txt").readlines()]
            attrs = [
                Attr(input_str=x)
                for x in open("frag/tests/data/attributes.txt").readlines()
            ]
        except IOError:
            nodes = [x for x in open("data/nodes.txt").readlines()]
            edges = [x.split() for x in open("data/edges.txt").readlines()]
            attrs = [Attr(input_str=x) for x in open("data/attributes.txt").readlines()]
        node_holder = NodeHolder(iso_flag=True)
        node_holder = build_network(attrs, node_holder)
        # Create the nodes and test with output
        self.assertEqual(len(node_holder.node_list), len(nodes))
        # This doesn't work yet(we get 3687 edges - should be 3691
        # Close enough - and the output looks right...
        self.assertEqual(len(node_holder.get_edges()), 3687)

    def test_generate_nodes_non_iso(self):
        """
        Test we can generate nodes for the basic data.
        :return:
        """
        try:
            nodes = [x for x in open("frag/tests/data/nodes.txt").readlines()]
            edges = [x.split() for x in open("frag/tests/data/edges.txt").readlines()]
            attrs = [
                Attr(input_str=x)
                for x in open("frag/tests/data/attributes.txt").readlines()
            ]
        except IOError:
            nodes = [x for x in open("data/nodes.txt").readlines()]
            edges = [x.split() for x in open("data/edges.txt").readlines()]
            attrs = [Attr(input_str=x) for x in open("data/attributes.txt").readlines()]
        node_holder = NodeHolder(iso_flag=False)
        node_holder = build_network(attrs, node_holder)
        # Create the nodes and test with output
        self.assertEqual(len(node_holder.node_list), len(nodes))
        # This doesn't work yet(we get 3687 edges - should be 3691
        # Close enough - and the output looks right...
        self.assertEqual(len(node_holder.get_edges()), 3687)

    def test_compare_iso_non_iso(self):
        """
        Test that the iso flag makes a difference.
        :return:
        """
        input_smis = ["C#CC(C)(C)NC[C@]1(O)CCCN2CCCC[C@@H]21"]
        test_iso_node_list = [
            "C#CC(C)(C)NC",
            "OC1CCCN2CCCCC12",
            "O",
            "C#CC(C)(C)NCC1CCCN2CCCCC12",
            "C#CC(C)(C)NC[C@]1(O)CCCN2CCCC[C@@H]21",
            "C1CCN2CCCCC2C1",
            "C#CC(C)(C)NC.O",
        ]
        test_non_iso_node_list = [
            "C#CC(C)(C)NC",
            "OC1CCCN2CCCCC12",
            "O",
            "C#CC(C)(C)NCC1CCCN2CCCCC12",
            "C#CC(C)(C)NCC1(O)CCCN2CCCCC21",
            "C1CCN2CCCCC2C1",
            "C#CC(C)(C)NC.O",
        ]
        test_iso_edge_list = [
            "EDGE C#CC(C)(C)NC[C@]1(O)CCCN2CCCC[C@@H]21 OC1CCCN2CCCCC12 FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]|RING|OC1([Xe])CCCN2CCCCC21|O[C@@]1([100Xe])CCCC2CCCC[C@@H]21",
            "EDGE OC1CCCN2CCCCC12 O RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12|FG|O[Xe]|O[100Xe]",
            "EDGE C#CC(C)(C)NC[C@]1(O)CCCN2CCCC[C@@H]21 C#CC(C)(C)NC.O RING|[Xe]C1([Xe])CCCN2CCCCC21|[100Xe][C@]1([101Xe])CCCC2CCCC[C@@H]21|FG|C#CC(C)(C)NC[Xe].O[Xe]|CCC(C)(C)NC[100Xe].O[101Xe]",
            "EDGE OC1CCCN2CCCCC12 C1CCN2CCCCC2C1 FG|O[Xe]|O[100Xe]|RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12",
            "EDGE C#CC(C)(C)NC.O C#CC(C)(C)NC FG|O|O|FG|C#CC(C)(C)NC|CCC(C)(C)NC",
            "EDGE C#CC(C)(C)NC[C@]1(O)CCCN2CCCC[C@@H]21 C#CC(C)(C)NCC1CCCN2CCCCC12 FG|O[Xe]|O[101Xe]|RING|C#CC(C)(C)NCC1([Xe])CCCN2CCCCC21|CCC(C)(C)NC[C@@]1([101Xe])CCCC2CCCC[C@@H]21",
            "EDGE C#CC(C)(C)NCC1CCCN2CCCCC12 C1CCN2CCCCC2C1 FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]|RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12",
            "EDGE C#CC(C)(C)NC.O O FG|C#CC(C)(C)NC|CCC(C)(C)NC|FG|O|O",
            "EDGE C#CC(C)(C)NCC1CCCN2CCCCC12 C#CC(C)(C)NC RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12|FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]",
        ]

        test_non_iso_edge_list = [
            "EDGE C#CC(C)(C)NCC1(O)CCCN2CCCCC21 C#CC(C)(C)NCC1CCCN2CCCCC12 FG|O[Xe]|O[101Xe]|RING|C#CC(C)(C)NCC1([Xe])CCCN2CCCCC21|CCC(C)(C)NCC1([101Xe])CCCC2CCCCC21",
            "EDGE OC1CCCN2CCCCC12 C1CCN2CCCCC2C1 FG|O[Xe]|O[100Xe]|RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12",
            "EDGE C#CC(C)(C)NCC1(O)CCCN2CCCCC21 C#CC(C)(C)NC.O RING|[Xe]C1([Xe])CCCN2CCCCC21|[100Xe]C1([101Xe])CCCC2CCCCC21|FG|C#CC(C)(C)NC[Xe].O[Xe]|CCC(C)(C)NC[100Xe].O[101Xe]",
            "EDGE OC1CCCN2CCCCC12 O RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12|FG|O[Xe]|O[100Xe]",
            "EDGE C#CC(C)(C)NCC1CCCN2CCCCC12 C1CCN2CCCCC2C1 FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]|RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12",
            "EDGE C#CC(C)(C)NC.O O FG|C#CC(C)(C)NC|CCC(C)(C)NC|FG|O|O",
            "EDGE C#CC(C)(C)NCC1(O)CCCN2CCCCC21 OC1CCCN2CCCCC12 FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]|RING|OC1([Xe])CCCN2CCCCC21|OC1([100Xe])CCCC2CCCCC21",
            "EDGE C#CC(C)(C)NC.O C#CC(C)(C)NC FG|O|O|FG|C#CC(C)(C)NC|CCC(C)(C)NC",
            "EDGE C#CC(C)(C)NCC1CCCN2CCCCC12 C#CC(C)(C)NC RING|[Xe]C1CCCN2CCCCC12|[100Xe]C1CCCC2CCCCC12|FG|C#CC(C)(C)NC[Xe]|CCC(C)(C)NC[100Xe]",
        ]

        attrs = [Attr(input_smi) for input_smi in input_smis]
        node_holder = NodeHolder(iso_flag=False)
        node_holder = build_network(attrs, node_holder)
        non_iso_node_list = [x.SMILES for x in node_holder.node_list]
        non_iso_edge_list = [str(x) for x in node_holder.edge_list]
        self.assertListEqual(sorted(non_iso_node_list), sorted(test_non_iso_node_list))
        self.assertListEqual(sorted(non_iso_edge_list), sorted(test_non_iso_edge_list))
        node_holder = NodeHolder(iso_flag=True)
        node_holder = build_network(attrs, node_holder)
        iso_node_list = [x.SMILES for x in node_holder.node_list]
        iso_edge_list = [str(x) for x in node_holder.edge_list]
        self.assertListEqual(sorted(iso_node_list), sorted(test_iso_node_list))
        self.assertListEqual(sorted(iso_edge_list), sorted(test_iso_edge_list))

    def test_decorate(self):
        """
        Test we can decorate a series of input SMILEs
        :return:
        """
        input_data = ["Oc1ccc(cc1)c2ccccc2", "c1ccccc1", "c1ccncc1", "c1cccnc1"]
        output_data = [
            [
                "Oc1ccc(-c2ccccc2[At])cc1",
                "Oc1ccc(-c2ccccc2)c([At])c1",
                "Oc1ccc(-c2ccc([At])cc2)cc1",
                "Oc1ccc(-c2ccccc2)cc1[At]",
                "Oc1ccc(-c2cccc([At])c2)cc1",
            ],
            ["[At]c1ccccc1"],
            ["[At]c1cccnc1", "[At]c1ccccn1", "[At]c1ccncc1"],
            ["[At]c1cccnc1", "[At]c1ccccn1", "[At]c1ccncc1"],
        ]
        for i, smi in enumerate(input_data):
            self.assertListEqual(list(decorate_smi(smi).keys()), output_data[i])

    def test_comb_index(self):
        """
        Test we combine indices
        :return:
        """
        input_data = [(12, 19), (6, 14), (99, 98), (4, 0)]
        output_data = [2012, 1506, 9999, 104]
        for i, data in enumerate(input_data):
            self.assertEqual(get_comb_index(data[0], data[1]), output_data[i])
            self.assertTupleEqual(ret_comb_index(output_data[i]), data)
            self.assertTupleEqual(
                ret_comb_index(get_comb_index(data[0], data[1])), data
            )

    def test_ring_ring_smi(self):
        input_smi = "CC(=O)NC=1C=CC(=CC1)C2=CSC(N)=N2"
        res = deletion_linker_smi("CC(=O)NC=1C=CC(=CC1)C2=CSC(N)=N2")
        self.assertListEqual(
            [Chem.MolToSmiles(x, isomericSmiles=True) for x in res[0]],
            ["Nc1nc(-c2ccc([100Xe])cc2)cs1", "CC(=O)Nc1ccc(-c2csc([102Xe])n2)cc1"],
        )
        self.assertListEqual(
            [Chem.MolToSmiles(x, isomericSmiles=True) for x in res[1]],
            ["CC(=O)Nc1ccc([1107Xe])cc1.Nc1nc([1107Xe])cs1"],
        )
        self.assertListEqual(
            [Chem.MolToSmiles(x, isomericSmiles=True) for x in res[2]],
            ["CC(=O)N[100Xe].Nc1nc([101Xe])cs1", "CC(=O)Nc1ccc([101Xe])cc1.N[102Xe]"],
        )

    def test_ring_ring_mol(self):
        input_sd = """
     RDKit          3D

 16 17  0  0  0  0  0  0  0  0999 V2000
  -10.1430  -10.6060  -10.9420 C   0  0  0  0  0  0  0  0  0  0  0  0
  -11.4350  -10.5270  -11.7010 C   0  0  0  0  0  0  0  0  0  0  0  0
  -12.5150  -10.4940  -11.1270 O   0  0  0  0  0  0  0  0  0  0  0  0
  -11.3000  -10.4840  -13.0450 N   0  0  0  0  0  0  0  0  0  0  0  0
  -12.2950  -10.4260  -14.0500 C   0  0  0  0  0  0  0  0  0  0  0  0
  -11.8760  -10.0530  -15.3220 C   0  0  0  0  0  0  0  0  0  0  0  0
  -12.7770   -9.9750  -16.3610 C   0  0  0  0  0  0  0  0  0  0  0  0
  -14.1220  -10.2670  -16.1660 C   0  0  0  0  0  0  0  0  0  0  0  0
  -14.5380  -10.6530  -14.8930 C   0  0  0  0  0  0  0  0  0  0  0  0
  -13.6380  -10.7330  -13.8470 C   0  0  0  0  0  0  0  0  0  0  0  0
  -15.0650  -10.1850  -17.2930 C   0  0  0  0  0  0  0  0  0  0  0  0
  -16.4170  -10.0960  -17.2170 C   0  0  0  0  0  0  0  0  0  0  0  0
  -17.1150   -9.9510  -18.7790 S   0  0  0  0  0  0  0  0  0  0  0  0
  -15.5150  -10.0790  -19.4610 C   0  0  0  0  0  0  0  0  0  0  0  0
  -15.3280  -10.0290  -20.7950 N   0  0  0  0  0  0  0  0  0  0  0  0
  -14.5560  -10.2150  -18.5880 N   0  0  0  0  0  0  0  0  0  0  0  0
  2  1  1  0
  3  2  2  0
  4  2  1  0
  5  4  1  0
  6  5  2  0
  7  6  1  0
  8  7  2  0
  9  8  1  0
 10  9  2  0
 10  5  1  0
 11  8  1  0
 12 11  2  0
 13 12  1  0
 14 13  1  0
 15 14  1  0
 16 14  2  0
 16 11  1  0
M  END

"""
        res = del_link_coord(input_sd)
        values = res["linkers"]["CC(=O)Nc1ccc([Xe])cc1.Nc1nc([Xe])cs1"]
        self.assertEqual(len(values), 4)
        self.assertEqual(type(values[0][2]), float)
        self.assertEqual(type(values[2][1]), float)



from frag.utils.vector_utils import get_exit_vector_for_xe_smi
import unittest
class VectorTest(unittest.TestCase):

    def test_xe_smi(self):
        # Some simple test data - these should all be zero
        input_data = ["[100Xe]c1ccc([101Xe])cc1","[101Xe]c1ccccc1"]
        output_data = [{100:0.0,101: 0.0,},{101:0.0}]
        # Now iterate over
        for i,smi in enumerate(input_data):
            ans = get_exit_vector_for_xe_smi(smi)
            test_dict = output_data[i]
            for key in test_dict:
                self.assertAlmostEqual(ans[key],test_dict[key],2)





from frag.alysis.cluster import dp_means
from frag.utils.parser import parse_ligand_ph4s

PH4_LAMBDA = 1.0
C_OF_M_LAMBDA = 6.0


def build_type_dict(mol_ph4_list, identifiers):
    type_dict = {}
    for i, mol in enumerate(mol_ph4_list):
        for ph4 in mol:
            x = ph4[0]
            y = ph4[1]
            z = ph4[2]
            ph4_type = ph4[3]
            if ph4_type in type_dict:
                type_dict[ph4_type]["coords"].append((x, y, z))
                type_dict[ph4_type]["mols"].append(identifiers[i])
            else:
                type_dict[ph4_type] = {"coords": [(x, y, z)], "mols": [identifiers[i]]}
    return type_dict


def map_cluster(dp_means_cluster, mol_id_list):
    """

    :param dp_means_cluster:
    :param mol_id_list:
    :return:
    """
    out_dict = {}
    for cluster in dp_means_cluster.clusters:
        out_dict[cluster] = {
            "centre_of_mass": dp_means_cluster.clusters[cluster],
            "mol_ids": [],
        }
    for i, cluster_id in enumerate(dp_means_cluster.dataClusterId):
        out_dict[cluster_id]["mol_ids"].append(mol_id_list[i])
    return out_dict


def cluster_dp(vect_list, lam, mol_list):
    """
    Perform a DP Means clustering.
    :param vect_list: a list of lists of vectors
    :param lam: the clustering parameters
    :param mol_list: the molecular identifers in the same order as the list of vectors
    :return: a dictionary of the form {cluster_id: {centre_of_mass: (x,y,z), mol_ids: [1,5,12]}}
    """
    return map_cluster(dp_means(vect_list, lam), mol_list)


def run_lig_cluster(mols, identifiers):
    """
    Cluster a list of molecules
    :param mols: the input list of RDKit molecules
    :param identifiers: the corresponding list of identifiers
    :return:
    """
    # First we get the list of mols with their Ph4s
    mol_ph4_list = parse_ligand_ph4s(mols)
    # Then we build a dict of type: coords: [coords list], mols: [mol_index]
    type_dict = build_type_dict(mol_ph4_list, identifiers)
    # Then we cluster coords
    clusters = {}
    for ph4_type in type_dict:
        if ph4_type == "c_of_m":
            clusters[ph4_type] = cluster_dp(
                type_dict[ph4_type]["coords"],
                C_OF_M_LAMBDA,
                type_dict[ph4_type]["mols"],
            )
        else:
            clusters[ph4_type] = cluster_dp(
                type_dict[ph4_type]["coords"], PH4_LAMBDA, type_dict[ph4_type]["mols"]
            )
    return clusters



import pickle, time
from rdkit.Chem import rdSubstructLibrary
from rdkit import Chem
import tqdm

def generate_fps(input_smi):
    # start by building the fingerprint cache
    t1 = time.time()
    with open(input_smi, 'r') as inf:
        ls = [[x.split()[0],x.split()[1]] for x in inf]
        ls.pop(0)
        with open(input_smi.replace(".smi",".pkl"), 'wb+') as pklf:
            for i, (smi,chemblid) in enumerate(ls):
                m = Chem.MolFromSmiles(smi, sanitize=False)
                fp = Chem.PatternFingerprint(m)
                pickle.dump(fp, pklf)
                if not (i + 1) % 10000:
                    print("Done", i + 1)
    t2=time.time()
    print("That took %.2f seconds. The output has %d molecules."%(t2-t1,i))



def read_in_lib(input_smi):
    t1=time.time()
    mols = rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    fps = rdSubstructLibrary.PatternHolder()
    with open(input_smi,'r') as inf:
        ls = [x.split() for x in inf]
        ls.pop(0)
        with open(input_smi.replace(".smi",".pkl"),'rb') as pklf:
            for l in tqdm.tqdm(ls):
                smi = l[1]
                mols.AddSmiles(smi)
                fp = pickle.load(pklf)
                fps.AddFingerprint(fp)
    library = rdSubstructLibrary.SubstructLibrary(mols,fps)
    t2=time.time()
    print("That took %.2f seconds. The library has %d molecules."%(t2-t1,len(library)))
    return library



def align_structure(structure_one, other_structures, align_residues):
    """
    Need to be able to robustly align two structures (normally > 10 to a reference structure).
    Maybe wrap Biojava (or even NGL) for this - not sure.
    :return:
    """
    pass


def extract_ligand(protein, ligand_res_id, ligand_smiles):
    """
    Need to be able to robustly extract a ligand from a structure.
    RDKit + some PDB weirdness should work here.
    :return:
    """
    pass


def find_interactions(protein, ligand):
    """
    Need to be able to robustly extract a ligand from a structure.
    This is about defining the SMARTS patterns / angles / distances to define interactions.
    :return:
    """
    pass


# TODO - I need corner cases - like tons of them from pharma - and what they expect to see. Then we run into pipeline and test.
# TODO - Aim is to have this clean and distributed by September.



class Connect(object):

    def __init__(
        self, username, password="", url="https://sso.apps.xchem.diamond.ac.uk"
    ):
        self.username = username
        self.password = password
        self.url = url
        self.token = None

    def get_token(self):
        """token=$(curl -kL -H 'Content-Type: application/x-www-form-urlencoded' -d 'grant_type=client_credentials' -d 'client_id=squonk-notebook' -d 'client_secret=dpyyjLR2dKUPBw2LfT3Ml3m4GtdeOIU0' https://sso.apps.xchem.diamond.ac.uk/auth/realms/xchem/protocol/openid-connect/token"""
        # TODO Where do we get the client secret from???
        url_extension = "/auth/realms/xchem/protocol/openid-connect/token"
        # Parse response - | grep -Po '(?<="access_token":")[^"]*')"
        token = ""
        self.token = token

    def get_jobs(self):
        """curl -L -H "Authorization: bearer $token" -H "SquonkUsername: user101" http://jobexecutor.squonk.svc:8080"""
        url_extension = "jobexecutor/rest/v1/jobs/"

    def post_job(self):
        """curl -L -H "Authorization: bearer $token" -H "SquonkUsername: user101" http://jobexecutor.squonk.svc:8080"""
        # TODO - quite extensive job preparation needed here - should link to DSD descriptions - and bespoke Python Classes
        url_extension = "jobexecutor/rest/v1/jobs/"

    # These are quite straightforward
    def poll_status(self, job_id):
        """curl -L -H "Authorization: bearer $token" -H "SquonkUsername: user101" http://jobexecutor.squonk.svc:8080/jobexecutor/rest/v1/jobs/<job-id>/status"""
        # Deal here with different options for returned jobs - e.g. RESULTS_READY etc

    def get_results(self, job_id):
        """curl -L -H "Authorization: bearer $token" -H "SquonkUsername: user101" http://jobexecutor.squonk.svc:8080/jobexecutor/rest/v1/jobs/<job-id>/results"""

    def delete_job(self, job_id):
        """curl -L -X DELETE -H "Authorization: bearer $token" -H "SquonkUsername: user101" http://jobexecutor.squonk.svc:8080/jobexecutor/rest/v1/jobs/<job-id>"""



from rdkit import Chem
from rdkit.Chem import AllChem


def get_core_mol(three_d_mol, core_mol):
    repl_sidechains = AllChem.ReplaceSidechains(three_d_mol, core_mol)
    if repl_sidechains:
        return AllChem.DeleteSubstructs(repl_sidechains, Chem.MolFromSmiles("*"))


def gen_conf_from_vector(input_mol_block, vector, elaborated_smiles):
    # Get the mol
    output_mol = Chem.MolFromSmiles(elaborated_smiles)
    three_d_mol = Chem.MolFromMolBlock(input_mol_block)
    # Get the shared core
    core = get_core_mol(
        three_d_mol,
        Chem.RemoveHs(Chem.MolFromSmiles(vector.split("_")[0].replace("Xe", "H"))),
    )
    if core:
        return Chem.MolToMolBlock(AllChem.ConstrainedEmbed(output_mol, core))


def generate_confs_for_vector(input_vector, input_smiles, input_mol_block):
    confs = []
    for elaborated_smiles in input_smiles:
        confs.append(
            gen_conf_from_vector(input_mol_block, input_vector, elaborated_smiles)
        )
    return confs


def generate_confs_for_vectors(input_vectors, input_mol_block):
    confs = {}
    for vector in input_vectors:
        confs[vector] = generate_confs_for_vector(
            vector, input_vectors[vector], input_mol_block
        )
    return confs


if __name__ == "__main__":
    generate_confs_for_vectors(
        {"O=C(Nc1ccc([Xe])nc1)N1CCNCC1_1_ADDITION": ["COc1ccc(NC(=O)N2CCNCC2)cn1"]},
        "\n     RDKit          3D\n\n 15 16  0  0  0  0  0  0  0  0999 V2000\n   -8.0100   -6.1770   18.7960 O   0  0  0  0  0  0  0  0  0  0  0  0\n   -7.8350   -7.3610   19.1060 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.7970   -8.0980   18.5710 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.0500   -7.8150   17.3890 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -6.2220   -6.6910   16.5800 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -5.5070   -6.4830   15.4790 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -4.5740   -7.3730   15.1470 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -4.3180   -8.4870   15.8900 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -5.0700   -8.7240   17.0200 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -8.6530   -7.9840   20.0040 N   0  0  0  0  0  0  0  0  0  0  0  0\n   -8.8660   -9.4290   20.1410 C   0  0  0  0  0  0  0  0  0  0  0  0\n  -10.3240   -9.8000   19.9120 C   0  0  0  0  0  0  0  0  0  0  0  0\n  -11.2290   -8.6780   20.2340 N   0  0  0  0  0  0  0  0  0  0  0  0\n  -10.7100   -7.8960   21.3670 C   0  0  0  0  0  0  0  0  0  0  0  0\n   -9.4210   -7.2020   20.9820 C   0  0  0  0  0  0  0  0  0  0  0  0\n  1  2  2  0\n  2  3  1  0\n  2 10  1  0\n  3  4  1  0\n  4  5  2  0\n  4  9  1  0\n  5  6  1  0\n  6  7  2  0\n  7  8  1  0\n  8  9  2  0\n 10 11  1  0\n 10 15  1  0\n 11 12  1  0\n 12 13  1  0\n 13 14  1  0\n 14 15  1  0\nM  END\n",
    )



import json

from frag.utils.network_utils import write_results


def write_all_out(out_d):
    for key in out_d:
        img = write_results(out_d[key])
        out_file = open(key+".svg","w")
        out_file.write(img)

if __name__ == "__main__":
    write_all_out(json.load(open("data.json")))




import unittest
from frag.alysis.cluster import DPMeans,KMeans


class ParserTest(unittest.TestCase):

    def test_dp_cluster(self):
        """
        Basic test to show clustering works.
        :return:
        """
        dp = DPMeans([(1.0,1.0,1.0),(3.0,3.0,3.0)], 1.0, 0, False)
        dp.run()
        self.assertEqual(len(dp.clusters),2)

    def test_kmean_cluster(self):
        """
        Simple test to show clustering works.
        :return:
        """
        kmean = KMeans([(1.0,1.0,1.0),(3.0,3.0,3.0)], 2, 0, False)
        kmean.run()
        self.assertEqual(len(kmean.clusters),2)


import os
import shutil

def do_for_dir(input_dir,num):
    os.chdir(input_dir)
    prop_dict = {"nodes.txt":[None,"smiles:ID(F2)", "hac:INT", "chac:INT", "osmiles"],
    "edges.txt":[None,":START_ID(F2)",":END_ID(F2)","label"],
    }
    attr_smis = []
    attr_ids = []
    for x in open("attributes.txt").readlines():
        attr_smis.append(x.split()[1])
        attr_ids.append(x.split()[3])

    for f_name in prop_dict:
        out_f = open(f_name.replace(".txt",".csv"),"w")
        for line in open(f_name).readlines():
            line_spl = line.split()
            out_l = []
            for i,x in enumerate(prop_dict[f_name]):
                if x:
                    out_l.append(line_spl[i])
            if f_name == "nodes.txt":
                if line_spl[1] in attr_smis:
                    cmpd_id = attr_ids[attr_smis.index(line_spl[1])]
                    out_l.append(cmpd_id)
                    # This is where we can add tags - like CHEAP - EXPENSIVE
                    out_l.append("EM;MOL;F2")
                else:
                    out_l.append("")
                    out_l.append("F2")
                out_f.write(",".join(out_l)+"\n")
            elif f_name =="edges.txt":
                out_f.write(",".join(out_l)+"\n")
        out_f.flush()
        out_f.close()
    # Copy this back and add to final
    shutil.copy("edges.csv","../edges_"+str(num)+".csv")
    shutil.copy("nodes.csv","../nodes_"+str(num)+".csv")
    os.chdir("../")

node_list = ["nodes-header.csv"]
edge_list = ["edges-header.csv"]
for i in range(19):
    print(i)
    if i==2:
        continue
    with open("edges-header.csv","w") as out_f:
        out_f.write(",".join([x for x in [":START_ID(F2)",":END_ID(F2)","label"]]))
    with open("nodes-header.csv", "w") as out_f:
        out_f.write(",".join([x for x in ["smiles:ID(F2)", "hac:INT", "chac:INT",
                                          "osmiles", "cmpd_id", ":LABEL"] if x]) + "\n")
    do_for_dir("ENA_SCREEN_"+str(i+1),i+1)
    node_list.append("nodes_"+str(i+1)+".csv")
    edge_list.append("edges_"+str(i+1)+".csv")
print(" ".join(["/var/lib/neo4j/bin/neo4j-admin import","--database", "new.db", "--nodes",'"'+",".join(node_list)+'"',
                "--relationships:F2EDGE",'"'+",".join(edge_list)+'"']))



#!/usr/bin/env python
import sys
import random
import math


class KMeans(object):

    def __init__(self, _X, _k, _xVal = 0, _stop=False):
        # X is sample size lists of dim length
        #
        # _xVal is the number of records to hold out cross-validation.
        # To use this you must randomize input data!
        #
        # Setting _stop=True causes iteration to stop when out of cross-validate
        # error starts to rise.
        #
        self.nFeatures = len(_X[0])
        self.xValSize = _xVal
        self.allSize = len(_X)
        self.size = self.allSize - self.xValSize
        self.X = _X
        self.k = _k
        self.stop = _stop
        # Initialize group memebership
        self.dataClusterId = [-1 for i in range(0, self.allSize)] # index of group for each data pair
        self.clusters = {}
        idx = 0
        # initialize to k random data points
        # don't assign x-val as a strat center
        for i in random.sample(list(range(0, self.size)), self.k):
            self.clusters[idx] = self.X[i]
            idx += 1
        # output records
        self.record = []
        self.errorRecord = []

    def dSquared(self, x, y):
        dist2 = 0.0
        for j,k in zip(x,y):
            dist2 += (j - k)**2
        return dist2

    def error(self):
        res = 0.0
        for i in range(0, self.size):
            res += self.dSquared(self.X[i], self.clusters[self.dataClusterId[i]])
        # error on non training data
        res1 = 0.0
        err1 = 0.0
        for i in range(self.size, self.allSize):
            res1 += self.dSquared(self.X[i], self.clusters[self.dataClusterId[i]])
        if res1 > 0.0:
            err1 = res1/self.xValSize
        return res/self.size, err1

    def nearestCluster(self, x):
        cmin = sys.maxsize
        cidx = -sys.maxsize
        for j in self.clusters:
            dist = math.sqrt(self.dSquared(x, self.clusters[j]))
            if dist < cmin:  # record closest centroid
                cmin = dist
                cidx = j
        return cidx, cmin

    def assign(self):
        for i in range(0, self.allSize):
            self.dataClusterId[i], dmin = self.nearestCluster(self.X[i])

    def updateClusters(self):
        ctemp = {} # dim sums by cluster
        for j in range(0, self.k):
            ctemp[j] = []
            for k in range(0, self.nFeatures):
                ctemp[j].append(0.0) # init sums
            ctemp[j].append(0) # init counter
        # only calculate clusters on training, not cross-validation set
        for i in range(0,self.size):
            for j in range(0, self.nFeatures):
                ctemp[self.dataClusterId[i]][j] += self.X[i][j]
            ctemp[self.dataClusterId[i]][self.nFeatures] += 1 # count
        for c in self.clusters:
            if ctemp[c][self.nFeatures] != 0:
                self.clusters[c] = [ ctemp[c][k]/ctemp[c][self.nFeatures] for k in range(0,self.nFeatures)]
            else:
                # no members in this cluster
                pass
        return

    def run(self, nmax = 100, eps = 1e-7):
        prev = 0.0
        prevXVal = float(sys.maxsize)
        for iter in range(0,nmax):
            # update assignments
            self.assign()
            # calculate error
            err, errXVal = self.error()
            #
            if self.stop and errXVal - prevXVal >= 0.0:
                sys.stderr.write("Cross-validation error increasing at step %d\n"%iter)
                break
            prevXVal = errXVal
            #
            if abs(err-prev) < eps:
                sys.stderr.write("Tolerance reached at step %d\n"%iter)
                break
            prev = err
            # going on...
            self.errorRecord.append((iter, err, errXVal))
            self.output(str(iter))
            self.updateClusters()
        sys.stderr.write("Iterations completed: %d\n"%iter)
        sys.stderr.write("Final error: %f\n"%prev)
        sys.stderr.write("Final cross-validation error: %f\n"%prevXVal)
        # This is a step past stop if using cross-validation...
        self.output("Final")
        return err, errXVal

    def output(self, iter):
        for i in range(0,self.size):
            self.record.append([str(y) for y in self.X[i]] + [str(self.dataClusterId[i])] + ["Iter-%s"%iter])
        for i in range(self.size, self.allSize):
            self.record.append([str(y) for y in self.X[i]] + [str(self.dataClusterId[i])] + ["Xval-Iter-%s"%iter])
        for k in self.clusters:
            self.record.append([str(y) for y in self.clusters[k]] + [str(k)] + ["Cent-Iter-%s"%iter])

    def getOutput(self):
        for x in self.record:
            yield x

    def getErrors(self):
        for x in self.errorRecord:
            yield x


class DPMeans(KMeans):
    def __init__(self, _X, _lam = 1, _xVal = 0, _stop=False):
        # init k-means with 1 cluster
        KMeans.__init__(self, _X, 1, _xVal, _stop)
        self.lam = _lam

    def assign(self):
        for i in range(0, self.size):
            cidx, dmin = self.nearestCluster(self.X[i])
            if dmin > self.lam:
                self.k += 1
                self.clusters[self.k-1] = self.X[i]
                self.dataClusterId[i] = self.k - 1
            else:
                self.dataClusterId[i] = cidx
        # don't create new clusters on cross-validation data
        for i in range(self.size, self.allSize):
            self.dataClusterId[i], dmin = self.nearestCluster(self.X[i])

    def error(self):
        err, xValErr = KMeans.error(self)
        return err + self.lam * self.k, xValErr + self.lam * self.k


def dp_means(data, lamb):
    """
    Carry out DP Means clustering on a vector of vectors
    :param data: vector of vectors (all same length)
    :param lamb: the parameter - the distance between clusters
    :return:
    """
    dp = DPMeans(data, lamb, 0, False)
    dp.run()
    return dp



import argparse

if __name__ == "__main__":
    # Read in a SD or SMILES file - then write out into a specified directory
    parser = argparse.ArgumentParser(
        description="Annotate a Nodes.txt file with data from multiple attributes files"
    )
    parser.add_argument("--input_attrs")
    parser.add_argument("--input_lib")
    parser.add_argument("--input_nodes")
    args = parser.parse_args()
    attrs = []
    id = 0
    # Add multipl annotation readers
    attrs = open(args.input_attrs).readlines()
    attr_dict = {}
    for attr in attrs:
        line_spl = attr.split()
        attr_dict[line_spl[1]] = {"cmpd_id": line_spl[3], "annotation": "EM;MOL;F2"}
    nodes = open(args.input_nodes).readlines()
    out_f = open("nodes.csv", "w")
    out_f.write("smiles:ID(F2),hac:INT,chac:INT,osmiles,cmpd_id,:LABEL\n")
    for x in nodes:
        line_spl = x.split()
        line_new = line_spl[1:]
        if line_new[0] in attr_dict:
            line_new.append(attr_dict[line_new[0]]["cmpd_id"])
            line_new.append(attr_dict[line_new[0]]["annotation"])
        else:
            line_new.append("")
            line_new.append("F2")
        # Now write this out
        out_f.write(",".join(line_new) + "\n")



import random
from frag.utils.network_utils import write_results, get_driver, canon_input


class ReturnObject(object):

    def __init__(self, start_smi, end_smi, label, edge_count, change_frag, iso_label):
        """
        Build this object.
        :param start_smi:
        :param end_smi:
        :param label:
        :param frag_type:
        :param edge_count:
        """
        self.start_smi = start_smi
        self.end_smi = end_smi
        self.label = label
        self.iso_label = iso_label
        self.frag_type = None
        self.edge_count = edge_count
        self.change_frag = change_frag

    def __str__(self):
        out_list = [self.label, str(self.edge_count), self.frag_type]
        return "_".join(out_list)


def find_double_edge(tx, input_str):
    return tx.run(
        "MATCH (sta:F2 {smiles:$smiles})-[nm:F2EDGE]-(mid:F2)-[ne:F2EDGE]-(end:EM) where"
        " abs(sta.hac-end.hac) <= 3 and abs(sta.chac-end.chac) <= 1"
        " and sta.smiles <> end.smiles "
        "RETURN sta, nm, mid, ne, end "
        "order by split(nm.label, '|')[4], split(ne.label, '|')[2];",
        smiles=input_str,
    )


def find_triple_edge_growth(
    tx,
    input_str,
    heavy_atom_diff_min=6,
    heavy_atom_diff_max=10,
    mid_heavy_atom_diff_min=-1,
    mid_heavy_atom_diff_max=3,
):
    return tx.run(
        "MATCH (sta:F2 {smiles:$smiles})-[nm:F2EDGE]-(mid_one:F2)-[ne:F2EDGE]-(mid:EM)-[nm2:F2EDGE]-(mid_two:F2)-[ne2:F2EDGE]-(end:EM) where"
        " end.hac-sta.hac > $hacmin and end.hac-sta.hac <= $hacmax"
        " and mid.hac-sta.hac > $chacmin and mid.hac-sta.hac <= $chacmax"
        " and sta.smiles <> mid.smiles and sta.smiles <> end.smiles "
        " WITH collect("
        "{"
        "end: end.smiles,"
        "mid: mid.smiles,"
        "frag_one: mid_one.smiles,"
        "frag_two: mid_two.smiles"
        "}"
        ") AS edges"
        " RETURN edges",
        smiles=input_str,
        hacmin=heavy_atom_diff_min,
        hacmax=heavy_atom_diff_max,
        chacmin=mid_heavy_atom_diff_min,
        chacmax=mid_heavy_atom_diff_max,
    )


def add_follow_ups(tx, input_str):
    return tx.run(
        "MATCH (sta:F2 {smiles:$smiles})-[nm:F2EDGE]-(mid:F2)-[ne:F2EDGE]-(end:EM) where"
        " abs(sta.hac-end.hac) <= 3 and abs(sta.chac-end.chac) <= 1"
        " and sta.smiles <> end.smiles "
        " MERGE (end)-[:FOLLOW_UP]->(sta)",
        smiles=input_str,
    )


def find_proximal(tx, input_str):
    return tx.run(
        "match p = (n:F2{smiles:$smiles})-[nm]-(m:EM)"
        "where abs(n.hac-m.hac) <= 3 and abs(n.chac-m.chac) <= 1 "
        "return n, nm, m "
        "order by split(nm.label, '|')[4];",
        smiles=input_str,
    )


def find_custom(tx, input_str):
    return tx.run(input_str)


def get_type(r_group_form, sub_one, sub_two):
    if "." in r_group_form:
        if "C1" in sub_two:
            return "ring_linker"
        return "linker"
    if "C1" in sub_two:
        return "ring_replacement"
    return "replacement"


def define_double_edge_type(record):
    mol_one = record["sta"]
    label = str(record["ne"]["label"].split("|")[4])
    iso_label = str(record["ne"]["label"].split("|")[5])
    change_frag = str(record["ne"]["label"].split("|")[2])
    mol_two = record["mid"]
    mol_three = record["end"]
    diff_one = mol_one["hac"] - mol_two["hac"]
    diff_two = mol_two["hac"] - mol_three["hac"]
    ret_obj = ReturnObject(
        mol_one["smiles"], mol_three["smiles"], label, 2, change_frag, iso_label
    )
    if "." in label:
        ret_obj.frag_type = "LINKER"
    elif diff_one >= 0 and diff_two >= 0:
        ret_obj.frag_type = "DELETION"
    elif diff_one <= 0 and diff_two <= 0:
        ret_obj.frag_type = "ADDITION"
    else:
        ret_obj.frag_type = "REPLACE"
    return ret_obj


def define_proximal_type(record):
    """
    Define the type returned for proximal systems
    :param record:
    :return:
    """
    mol_one = record["n"]
    label = str(record["nm"]["label"].split("|")[4])
    iso_label = str(record["nm"]["label"].split("|")[5])
    change_frag = str(record["nm"]["label"].split("|")[2])
    mol_two = record["m"]
    ret_obj = ReturnObject(
        mol_one["smiles"], mol_two["smiles"], label, 1, change_frag, iso_label
    )
    if "." in label:
        ret_obj.frag_type = "LINKER"
    elif mol_one["hac"] - mol_two["hac"] > 0:
        ret_obj.frag_type = "DELETION"
    elif mol_one["hac"] - mol_two["hac"] < 0:
        ret_obj.frag_type = "ADDITION"
    else:
        ret_obj.frag_type = "REPLACE"
    return ret_obj


def organise(records, num_picks):
    out_d = {}
    smi_set = set()
    for rec in records:
        rec_key = str(rec)
        addition = {"change": rec.change_frag, "end": rec.end_smi}
        if rec_key in out_d:
            out_d[rec_key]["addition"].append(addition)
        else:
            out_d[rec_key] = {"vector": rec.iso_label, "addition": [addition]}
        smi_set.add(rec.end_smi)
    if num_picks:
        max_per_hypothesis = num_picks / len(out_d)
    out_smi = []
    for rec in out_d:
        # TODO here is the logic as to ordering replacements
        if num_picks:
            random.shuffle(out_d[rec]["addition"])
            out_d[rec]["addition"] = out_d[rec]["addition"][:max_per_hypothesis]
        else:
            out_d[rec]["addition"] = out_d[rec]["addition"]
        out_smi.extend(out_d[rec])
    return out_d


def get_picks(smiles, num_picks, graph_url="neo4j"):
    smiles = canon_input(smiles)
    driver = get_driver(graph_url)
    with driver.session() as session:
        records = []
        for record in session.read_transaction(find_proximal, smiles):
            ans = define_proximal_type(record)
            records.append(ans)
        for record in session.read_transaction(find_double_edge, smiles):
            ans = define_double_edge_type(record)
            records.append(ans)
        for label in list(set([x.label for x in records])):
            # Linkers are meaningless
            if "." in label:
                continue
        if records:
            orga_dict = organise(records, num_picks)
            return orga_dict
        else:
            print("Nothing found for input: " + smiles)


def get_full_graph(smiles, graph_url="neo4j", isomericSmiles=True):
    smiles = canon_input(smiles, isomericSmiles)
    driver = get_driver(graph_url)
    with driver.session() as session:
        records = []
        for record in session.read_transaction(find_proximal, smiles):
            ans = define_proximal_type(record)
            records.append(ans)
        for record in session.read_transaction(find_double_edge, smiles):
            ans = define_double_edge_type(record)
            records.append(ans)
        for label in list(set([x.label for x in records])):
            # Linkers are meaningless
            if "." in label:
                continue
        if records:
            orga_dict = organise(records, None)
            return orga_dict
        else:
            print("Nothing found for input: " + smiles)


def custom_query(query, graph_url="neo4j"):
    driver = get_driver(graph_url)
    records = []
    with driver.session() as session:
        for record in session.read_transaction(find_custom, query):
            records.append(record)
    return records


def write_picks(smiles, num_picks):
    img_dict = write_results(get_picks(smiles, num_picks))
    for key in img_dict:
        out_f = open(key + ".svg", "w")
        out_f.write(img_dict[key])



import uuid


class StructHolder(object):
    """
    A class to holad
    """

    def __init__(
        self, struct_id, resid_pdb=None, ligand=None, water_pdb=None, apo_pdb=None
    ):
        self.struct_id = struct_id
        self.water_pdb = water_pdb
        self.resid_pdb = resid_pdb
        self.ligand = ligand


class ClusterStuff(object):
    """
    A class to cluster stuff.
    Probably also need a write output function too. Once I've got my head around that...
    """

    def __init__(self, parser, lamb, cluster):
        self.parser = parser
        self.lamb = lamb
        self.cluster = cluster
        self.owner_list = []
        self.out_clusters = {}

    def run(self, input_data):
        # self.parser returns a list of Owner objects -
        self.owner_list = self.parser(input_data)
        # Now convert o
        data_set = self.owner_list_conv()
        for this_type in data_set:
            # Add clusters to objects
            self.out_clusters[this_type] = []
            self.add_clust_to_obj(
                self.cluster(data_set[this_type]["data"], this_type),
                data_set[this_type]["objects"],
                this_type,
            )

    def add_clust_to_obj(self, cluster_obj, object_list, type):
        """
        Given the output data -> assign an output cluster
        :return:
        """
        for i, cluster in enumerate(cluster_obj.clusters):
            new_cluster = Cluster(cluster_obj.clusters[cluster], type, i)
            self.out_clusters[type].append(new_cluster)
        for i, val in enumerate(cluster_obj.dataClusterId):
            this_clust = [x for x in self.out_clusters[type] if x.cluster == val][0]
            this_obj = object_list[i]
            this_obj.cluster = this_clust
            this_clust.object_list.append(this_obj)

    def write_output(self):
        """
        Write the output of the clustering to JSON or similar
        :return:
        """
        for type in self.out_clusters:
            self.out_clusters[type]

    def owner_list_conv(self):
        """
        Convert an owner list to a Dict of Type - cluster list.
        :return:
        """
        out_d = {}
        for owner in self.owner_list:
            for obj in owner.object_list:
                if obj.object_desc in out_d:
                    out_d[obj.object_desc]["data"].append(obj.value_array)
                    out_d[obj.object_desc]["objects"].append(obj)
                else:
                    out_d[obj.object_desc] = {
                        "data": [obj.value_array],
                        "pks": [obj.uuid],
                    }
        return out_d


class Object(object):

    def __init__(self, value_array, object_desc):
        """
        An Object will have a type and a value arrays
        :param value_array: e.g. x,y,z coords - what is clustered
        :param object_desc: e.g. "Water" or "H-bond acceptor"
        """
        self.value_array = value_array
        self.object_desc = object_desc
        self.uuid = str(uuid.uuid4())
        self.cluster = -1


class Owner(object):

    def __init__(self, object_list, title):
        """
        An Owner will own multiple objects. E.g. PDB 4CUP owns waters.
        :param object_list: the list of objects it owns
        :param title: the title of the object
        """
        self.object_list = object_list
        self.title = title


class Cluster(Object):

    def __init__(self, value_array, object_desc, index):
        """
        An Object will have a type and a value arrays
        :param value_array: e.g. x,y,z coords - what is clustered
        :param object_desc: e.g. "Water" or "H-bond acceptor"
        """
        self.value_array = value_array
        self.object_desc = object_desc
        self.uuid = str(uuid.uuid4())
        self.object_list = []
        self.cluster_ind = index
        Object.__init__(self, self.object_list, object_desc)

    def get_size(self):
        return len(self.object_list)



import argparse

from frag.network.query import write_picks

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description="Query a Database for a given SMILES")
    parser.add_argument("--smiles")
    parser.add_argument("--num_picks")
    args = parser.parse_args()
    num_picks = int(args.num_picks)
    smiles = args.smiles
    # Add this node
    write_picks(smiles, num_picks)



import requests

prot_code = "MURD-x0373"
r = requests.get(
    "https://fragalysis.apps.xchem.diamond.ac.uk/api/proteins/",
    params={"code": prot_code},
)
result = r.json()["results"][0]
pdb_url = result["pdb_info"]
pdb_id = result["id"]
r = requests.get(
    "https://fragalysis.apps.xchem.diamond.ac.uk/api/molecules/",
    params={"prot_id": pdb_id},
)
out_json = r.json()["results"][0]
sdf_info = out_json["sdf_info"]
out_f = open(prot_code + ".mol", "w")
out_f.write(sdf_info)
out_f.close()
# Now s
r = requests.get(pdb_url)
out_f = open(prot_code + "_apo.pdb", "w")
out_f.write(r.text)
out_f.close()



import argparse
import os

from frag.utils.network_utils import get_driver,add_attr,add_edge,add_node
from tqdm import tqdm


if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Upload Nodes, Edges and Attributes')
    parser.add_argument('--base_dir')
    args = parser.parse_args()
    driver = get_driver()

    with driver.session() as session:
        for line in tqdm(open(os.path.join(args.base_dir,"nodes.txt")).readlines()):
            session.write_transaction(add_node,line.split()[1],line.split()[2],line.split()[3],line.split()[4])
        for line in tqdm(open(os.path.join(args.base_dir,"edges.txt")).readlines()):
            session.write_transaction(add_edge,line.split()[1],line.split()[2],line.split()[3])
        for line in tqdm(open(os.path.join(args.base_dir,"attributes.txt")).readlines()):
            session.write_transaction(add_attr,line.split()[1],line.split()[3])


# Tools for merging, growing and linking fragments into hybrid queries
from frag.utils.rdkit_utils import RDKitPh4, RDKitAtom


def combine_ph4(
    mols, max_num_ph4s=4, min_cluster_size=2, smooth_param=0.5, smooth_grad=0.1
):
    """
    Combine a series of pharmacophores into a single query
    :param mols: a list of RDKit molecules
    :param max_num_ph4s: the maximum number of ph4s in a query
    :param min_cluster_size: the minimum size of a cluster for a point to be allowed
    :param smooth_param: the max allowed pertubation for an atom
    :param smooth_grad: the smoothing gradient for an atom
    :return: a series of possible RDKit Pharmacophore queries
    """
    rdkit_ph4 = RDKitPh4()
    molecule_ph4s = []
    for i, mol in enumerate(mols):
        ph4s = rdkit_ph4.generate_ph4_for_mol(mol)
        molecule_ph4s.append([i, mol, ph4s])
    # Cluster and remove points that don't fit into a given cluster

    # Combine these combinatorially and generate duplicates with the smoothing


def combine_atoms(
    mols, clash_param=1.5, min_cluster_size=2, smooth_param=0.5, smooth_grad=0.1
):
    """
    Combine a series of molecules into a single list of atoms - removing clashes
    :param mols: a list of RDKit molecules
    :param clash_param: the min distance between atoms
    :param min_cluster_size: the minimum size of a cluster for a point to be allowed
    :param smooth_param: the max allowed pertubation for an atom
    :param smooth_grad: the smoothing gradient for an atom
    :return: a series of possible combinations of atoms - e.g. as input into USRCAT
    """
    rdkit_atom = RDKitAtom()
    molecule_atoms = []
    for i, mol in enumerate(mols):
        atoms = rdkit_atom.generate_atoms_for_mol(mol)
        molecule_atoms.append([i, mol, atoms])


def bond_atoms(atom_list):
    """
    Attempt to merge allowed atoms given allowed rules and combinations.
    This will be based off an R group of a given fragment.
    Loss function to define a good molecule - More bonds, more interactions, less atoms.
    Simply as ratios:
    bonds-fulfilled / atom
    interactions / atom
    Need to keep the ratio as low
    Just train on single molecules, then try merging two, then more and more.
    :param atom_list: the list of allowed atoms, x,y,z and atom desc (atomic number and hybridization state)
    :return:
    """
    pass


def generate_exclusions(proteins):
    """
    Generate a series of exclusion spheres for some cavities
    :param proteins: a list of protein cavities
    :return: the combined exclusion spheres
    """
    pass
